@@ -1,7 +1,7 @@
 /*
  * irc-ctcp.c - IRC CTCP protocol
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -33,6 +33,7 @@
 #include "irc-channel.h"
 #include "irc-color.h"
 #include "irc-config.h"
+#include "irc-input.h"
 #include "irc-msgbuffer.h"
 #include "irc-nick.h"
 #include "irc-protocol.h"
@@ -79,23 +80,36 @@ const char *
 irc_ctcp_get_reply (struct t_irc_server *server, const char *ctcp)
 {
     struct t_config_option *ptr_option;
-    char option_name[512];
+    char option_name[512], *ctcp_lower;
 
-    snprintf (option_name, sizeof (option_name), "%s.%s", server->name, ctcp);
+    ctcp_lower = weechat_string_tolower (ctcp);
+    if (!ctcp_lower)
+        return NULL;
+
+    snprintf (option_name, sizeof (option_name),
+              "%s.%s", server->name, ctcp_lower);
 
     /* search for CTCP in configuration file, for server */
     ptr_option = weechat_config_search_option (irc_config_file,
                                                irc_config_section_ctcp,
                                                option_name);
     if (ptr_option)
+    {
+        free (ctcp_lower);
         return weechat_config_string (ptr_option);
+    }
 
     /* search for CTCP in configuration file */
     ptr_option = weechat_config_search_option (irc_config_file,
                                                irc_config_section_ctcp,
-                                               ctcp);
+                                               ctcp_lower);
     if (ptr_option)
+    {
+        free (ctcp_lower);
         return weechat_config_string (ptr_option);
+    }
+
+    free (ctcp_lower);
 
     /*
      * no CTCP reply found in config, then return default reply, or NULL
@@ -111,6 +125,7 @@ irc_ctcp_get_reply (struct t_irc_server *server, const char *ctcp)
 void
 irc_ctcp_display_request (struct t_irc_server *server,
                           time_t date,
+                          struct t_hashtable *tags,
                           const char *command,
                           struct t_irc_channel *channel,
                           const char *nick,
@@ -129,7 +144,7 @@ irc_ctcp_display_request (struct t_irc_server *server,
             server, nick, NULL, "ctcp",
             (channel) ? channel->buffer : NULL),
         date,
-        irc_protocol_tags (command, "irc_ctcp", NULL, address),
+        irc_protocol_tags (server, command, tags, "irc_ctcp", NULL, address),
         _("%sCTCP requested by %s%s%s: %s%s%s%s%s%s"),
         weechat_prefix ("network"),
         irc_nick_color_for_msg (server, 0, NULL, nick),
@@ -149,20 +164,27 @@ irc_ctcp_display_request (struct t_irc_server *server,
 
 void
 irc_ctcp_display_reply_from_nick (struct t_irc_server *server, time_t date,
+                                  struct t_hashtable *tags,
                                   const char *command, const char *nick,
-                                  const char *address, char *arguments)
+                                  const char *address, const char *arguments)
 {
-    char *pos_end, *pos_space, *pos_args, *pos_usec;
+    char *dup_arguments, *ptr_args, *pos_end, *pos_space, *pos_args, *pos_usec;
     struct timeval tv;
     long sec1, usec1, sec2, usec2, difftime;
 
-    while (arguments && arguments[0])
+    dup_arguments = strdup (arguments);
+    if (!dup_arguments)
+        return;
+
+    ptr_args = dup_arguments;
+
+    while (ptr_args && ptr_args[0])
     {
-        pos_end = strrchr (arguments + 1, '\01');
+        pos_end = strrchr (ptr_args + 1, '\01');
         if (pos_end)
             pos_end[0] = '\0';
 
-        pos_space = strchr (arguments + 1, ' ');
+        pos_space = strchr (ptr_args + 1, ' ');
         if (pos_space)
         {
             pos_space[0] = '\0';
@@ -171,7 +193,7 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server, time_t date,
             {
                 pos_args++;
             }
-            if (strcmp (arguments + 1, "PING") == 0)
+            if (weechat_strcasecmp (ptr_args + 1, "ping") == 0)
             {
                 pos_usec = strchr (pos_args, ' ');
                 if (pos_usec)
@@ -190,7 +212,8 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server, time_t date,
                         irc_msgbuffer_get_target_buffer (
                             server, nick, NULL, "ctcp", NULL),
                         date,
-                        irc_protocol_tags (command, "irc_ctcp", NULL, NULL),
+                        irc_protocol_tags (server, command, tags, "irc_ctcp",
+                                           NULL, NULL),
                         /* TRANSLATORS: %.3fs is a float number + "s" ("seconds") */
                         _("%sCTCP reply from %s%s%s: %s%s%s %.3fs"),
                         weechat_prefix ("network"),
@@ -198,11 +221,9 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server, time_t date,
                         nick,
                         IRC_COLOR_RESET,
                         IRC_COLOR_CHAT_CHANNEL,
-                        arguments + 1,
+                        ptr_args + 1,
                         IRC_COLOR_RESET,
                         (float)difftime / 1000000.0);
-
-                    pos_usec[0] = ' ';
                 }
             }
             else
@@ -211,19 +232,19 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server, time_t date,
                     irc_msgbuffer_get_target_buffer (
                         server, nick, NULL, "ctcp", NULL),
                     date,
-                    irc_protocol_tags (command, "irc_ctcp", NULL, address),
+                    irc_protocol_tags (server, command, tags, "irc_ctcp", NULL,
+                                       address),
                     _("%sCTCP reply from %s%s%s: %s%s%s%s%s"),
                     weechat_prefix ("network"),
                     irc_nick_color_for_msg (server, 0, NULL, nick),
                     nick,
                     IRC_COLOR_RESET,
                     IRC_COLOR_CHAT_CHANNEL,
-                    arguments + 1,
+                    ptr_args + 1,
                     IRC_COLOR_RESET,
                     " ",
                     pos_args);
             }
-            pos_space[0] = ' ';
         }
         else
         {
@@ -231,24 +252,23 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server, time_t date,
                 irc_msgbuffer_get_target_buffer (
                     server, nick, NULL, "ctcp", NULL),
                 date,
-                irc_protocol_tags (command, NULL, NULL, address),
+                irc_protocol_tags (server, command, tags, NULL, NULL, address),
                 _("%sCTCP reply from %s%s%s: %s%s%s%s%s"),
                 weechat_prefix ("network"),
                 irc_nick_color_for_msg (server, 0, NULL, nick),
                 nick,
                 IRC_COLOR_RESET,
                 IRC_COLOR_CHAT_CHANNEL,
-                arguments + 1,
+                ptr_args + 1,
                 "",
                 "",
                 "");
         }
 
-        if (pos_end)
-            pos_end[0] = '\01';
-
-        arguments = (pos_end) ? pos_end + 1 : NULL;
+        ptr_args = (pos_end) ? pos_end + 1 : NULL;
     }
+
+    free (dup_arguments);
 }
 
 /*
@@ -257,64 +277,104 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server, time_t date,
 
 void
 irc_ctcp_reply_to_nick (struct t_irc_server *server,
+                        struct t_hashtable *tags,
                         const char *command,
                         struct t_irc_channel *channel,
-                        const char *nick, const char *ctcp,
+                        const char *nick,
+                        const char *ctcp,
                         const char *arguments)
 {
-    struct t_hashtable *hashtable;
-    int number;
-    char hash_key[32];
-    const char *str_args;
-    char *str_args_color;
+    struct t_arraylist *list_messages;
+    int i, list_size;
+    char *msg_color, *dup_ctcp, *dup_ctcp_upper, *dup_args;
+    const char *ptr_message;
 
-    hashtable = irc_server_sendf (
+    dup_ctcp = NULL;
+    dup_ctcp_upper = NULL;
+    dup_args = NULL;
+    list_messages = NULL;
+
+    /*
+     * replace any "\01" by a space to prevent any firewall attack via
+     * nf_conntrack_irc (CVE-2022-2663)
+     */
+    dup_ctcp = weechat_string_replace (ctcp, "\01", " ");
+    if (!dup_ctcp)
+        goto end;
+
+    dup_ctcp_upper = weechat_string_toupper (dup_ctcp);
+    if (!dup_ctcp_upper)
+        goto end;
+
+    if (arguments)
+    {
+        /*
+         * replace any "\01" by a space to prevent any firewall attack via
+         * nf_conntrack_irc (CVE-2022-2663)
+         */
+        dup_args = weechat_string_replace (arguments, "\01", " ");
+        if (!dup_args)
+            goto end;
+    }
+
+    list_messages = irc_server_sendf (
         server,
-        IRC_SERVER_SEND_OUTQ_PRIO_LOW | IRC_SERVER_SEND_RETURN_HASHTABLE,
+        IRC_SERVER_SEND_OUTQ_PRIO_LOW | IRC_SERVER_SEND_RETURN_LIST,
         NULL,
         "NOTICE %s :\01%s%s%s\01",
-        nick, ctcp,
-        (arguments) ? " " : "",
-        (arguments) ? arguments : "");
+        nick,
+        dup_ctcp_upper,
+        (dup_args) ? " " : "",
+        (dup_args) ? dup_args : "");
+    if (!list_messages)
+        goto end;
 
-    if (hashtable)
+    if (weechat_config_boolean (irc_config_look_display_ctcp_reply))
     {
-        if (weechat_config_boolean (irc_config_look_display_ctcp_reply))
+        list_size = weechat_arraylist_size (list_messages);
+        for (i = 0; i < list_size; i++)
         {
-            number = 1;
-            while (1)
-            {
-                snprintf (hash_key, sizeof (hash_key), "args%d", number);
-                str_args = weechat_hashtable_get (hashtable, hash_key);
-                if (!str_args)
-                    break;
-                str_args_color = irc_color_decode (str_args, 1);
-                if (!str_args_color)
-                    break;
-                weechat_printf_tags (
-                    irc_msgbuffer_get_target_buffer (
-                        server, nick, NULL, "ctcp",
-                        (channel) ? channel->buffer : NULL),
-                    irc_protocol_tags (
-                        command,
-                        "irc_ctcp,irc_ctcp_reply,notify_none,no_highlight",
-                        NULL, NULL),
-                    _("%sCTCP reply to %s%s%s: %s%s%s%s%s"),
-                    weechat_prefix ("network"),
-                    irc_nick_color_for_msg (server, 0, NULL, nick),
-                    nick,
-                    IRC_COLOR_RESET,
-                    IRC_COLOR_CHAT_CHANNEL,
-                    ctcp,
-                    (str_args_color[0]) ? IRC_COLOR_RESET : "",
-                    (str_args_color[0]) ? " " : "",
-                    str_args_color);
-                free (str_args_color);
-                number++;
-            }
+            ptr_message = (const char *)weechat_arraylist_get (list_messages, i);
+            if (!ptr_message)
+                break;
+            msg_color = irc_color_decode (ptr_message, 1);
+            if (!msg_color)
+                break;
+            weechat_printf_date_tags (
+                irc_msgbuffer_get_target_buffer (
+                    server, nick, NULL, "ctcp",
+                    (channel) ? channel->buffer : NULL),
+                0,
+                irc_protocol_tags (
+                    server,
+                    command,
+                    tags,
+                    "irc_ctcp,irc_ctcp_reply,self_msg,notify_none,"
+                    "no_highlight",
+                    NULL, NULL),
+                _("%sCTCP reply to %s%s%s: %s%s%s%s%s"),
+                weechat_prefix ("network"),
+                irc_nick_color_for_msg (server, 0, NULL, nick),
+                nick,
+                IRC_COLOR_RESET,
+                IRC_COLOR_CHAT_CHANNEL,
+                dup_ctcp_upper,
+                (msg_color[0]) ? IRC_COLOR_RESET : "",
+                (msg_color[0]) ? " " : "",
+                msg_color);
+            free (msg_color);
         }
-        weechat_hashtable_free (hashtable);
     }
+
+end:
+    if (dup_ctcp)
+        free (dup_ctcp);
+    if (dup_ctcp_upper)
+        free (dup_ctcp_upper);
+    if (dup_args)
+        free (dup_args);
+    if (list_messages)
+        weechat_arraylist_free (list_messages);
 }
 
 /*
@@ -326,11 +386,10 @@ irc_ctcp_reply_to_nick (struct t_irc_server *server,
 char *
 irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
 {
-    char *res, *temp, *username, *realname;
-    const char *info;
+    char *res, *temp, *username, *realname, *info, *info2;
     time_t now;
     struct tm *local_time;
-    char buf[1024];
+    char buf[4096];
     struct utsname *buf_uname;
 
     /*
@@ -352,6 +411,8 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     info = weechat_info_get ("version_git", "");
     temp = weechat_string_replace (res, "$git", info);
     free (res);
+    if (info)
+        free (info);
     if (!temp)
         return NULL;
     res = temp;
@@ -363,13 +424,18 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
      *   0.4.0-dev (git: v0.3.9-104-g7eb5cc4)
      */
     info = weechat_info_get ("version_git", "");
+    info2 = weechat_info_get ("version", "");
     snprintf (buf, sizeof (buf), "%s%s%s%s",
-              weechat_info_get ("version", ""),
+              info2,
               (info && info[0]) ? " (git: " : "",
               (info && info[0]) ? info : "",
               (info && info[0]) ? ")" : "");
     temp = weechat_string_replace (res, "$versiongit", buf);
     free (res);
+    if (info)
+        free (info);
+    if (info2)
+        free (info2);
     if (!temp)
         return NULL;
     res = temp;
@@ -382,6 +448,8 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     info = weechat_info_get ("version", "");
     temp = weechat_string_replace (res, "$version", info);
     free (res);
+    if (info)
+        free (info);
     if (!temp)
         return NULL;
     res = temp;
@@ -393,6 +461,8 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     info = weechat_info_get ("date", "");
     temp = weechat_string_replace (res, "$compilation", info);
     free (res);
+    if (info)
+        free (info);
     if (!temp)
         return NULL;
     res = temp;
@@ -422,23 +492,27 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     }
 
     /*
-     * $site: WeeChat web site, example:
+     * $site: WeeChat website, example:
      *   https://weechat.org/
      */
     info = weechat_info_get ("weechat_site", "");
     temp = weechat_string_replace (res, "$site", info);
     free (res);
+    if (info)
+        free (info);
     if (!temp)
         return NULL;
     res = temp;
 
     /*
      * $download: WeeChat download page, example:
-     *   https://weechat.org/download
+     *   https://weechat.org/download/
      */
     info = weechat_info_get ("weechat_site_download", "");
     temp = weechat_string_replace (res, "$download", info);
     free (res);
+    if (info)
+        free (info);
     if (!temp)
         return NULL;
     res = temp;
@@ -450,9 +524,10 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     now = time (NULL);
     local_time = localtime (&now);
     setlocale (LC_ALL, "C");
-    strftime (buf, sizeof (buf),
-              weechat_config_string (irc_config_look_ctcp_time_format),
-              local_time);
+    if (strftime (buf, sizeof (buf),
+                  weechat_config_string (irc_config_look_ctcp_time_format),
+                  local_time) == 0)
+        buf[0] = '\0';
     setlocale (LC_ALL, "");
     temp = weechat_string_replace (res, "$time", buf);
     free (res);
@@ -464,9 +539,9 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
      * $username: user name, example:
      *   name
      */
-    username = weechat_string_eval_expression (
-        IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_USERNAME),
-        NULL, NULL, NULL);
+    username = irc_server_eval_expression (
+        server,
+        IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_USERNAME));
     if (username)
     {
         temp = weechat_string_replace (res, "$username", username);
@@ -481,9 +556,9 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
      * $realname: real name, example:
      *   John doe
      */
-    realname = weechat_string_eval_expression (
-        IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_REALNAME),
-        NULL, NULL, NULL);
+    realname = irc_server_eval_expression (
+        server,
+        IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_REALNAME));
     if (realname)
     {
         temp = weechat_string_replace (res, "$realname", realname);
@@ -510,7 +585,7 @@ irc_ctcp_dcc_filename_without_quotes (const char *filename)
     int length;
 
     length = strlen (filename);
-    if (length > 0)
+    if (length > 1)
     {
         if ((filename[0] == '\"') && (filename[length - 1] == '\"'))
             return weechat_strndup (filename + 1, length - 2);
@@ -524,13 +599,13 @@ irc_ctcp_dcc_filename_without_quotes (const char *filename)
 
 void
 irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
-                   const char *arguments, char *message)
+                   const char *arguments, const char *message)
 {
     char *dcc_args, *pos, *pos_file, *pos_addr, *pos_port, *pos_size;
     char *pos_start_resume, *filename;
     struct t_infolist *infolist;
     struct t_infolist_item *item;
-    char charset_modifier[256];
+    char charset_modifier[1024];
 
     if (!arguments || !arguments[0])
         return;
@@ -652,7 +727,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
 
         (void) weechat_hook_signal_send ("irc_dcc",
                                          WEECHAT_HOOK_SIGNAL_STRING,
-                                         message);
+                                         (void *)message);
 
         if (filename)
             free (filename);
@@ -748,7 +823,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
 
         (void) weechat_hook_signal_send ("irc_dcc",
                                          WEECHAT_HOOK_SIGNAL_STRING,
-                                         message);
+                                         (void *)message);
 
         if (filename)
             free (filename);
@@ -844,7 +919,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
 
         (void) weechat_hook_signal_send ("irc_dcc",
                                          WEECHAT_HOOK_SIGNAL_STRING,
-                                         message);
+                                         (void *)message);
 
         if (filename)
             free (filename);
@@ -955,7 +1030,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
 
         (void) weechat_hook_signal_send ("irc_dcc",
                                          WEECHAT_HOOK_SIGNAL_STRING,
-                                         message);
+                                         (void *)message);
 
         free (dcc_args);
     }
@@ -966,26 +1041,33 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
  */
 
 void
-irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
-               struct t_irc_channel *channel, const char *address,
-               const char *nick, const char *remote_nick, char *arguments,
-               char *message)
+irc_ctcp_recv (struct t_irc_server *server, time_t date,
+               struct t_hashtable *tags, const char *command,
+               struct t_irc_channel *channel, const char *target,
+               const char *address, const char *nick, const char *remote_nick,
+               const char *arguments, const char *message)
 {
-    char *pos_end, *pos_space, *pos_args;
+    char *dup_arguments, *ptr_args, *pos_end, *pos_space, *pos_args;
+    char *nick_color, *decoded_reply;
     const char *reply;
-    char *decoded_reply;
     struct t_irc_channel *ptr_channel;
     struct t_irc_nick *ptr_nick;
     int nick_is_me;
 
-    while (arguments && arguments[0])
+    dup_arguments = strdup (arguments);
+    if (!dup_arguments)
+        return;
+
+    ptr_args = dup_arguments;
+
+    while (ptr_args && ptr_args[0])
     {
-        pos_end = strrchr (arguments + 1, '\01');
+        pos_end = strrchr (ptr_args + 1, '\01');
         if (pos_end)
             pos_end[0] = '\0';
 
         pos_args = NULL;
-        pos_space = strchr (arguments + 1, ' ');
+        pos_space = strchr (ptr_args + 1, ' ');
         if (pos_space)
         {
             pos_space[0] = '\0';
@@ -997,7 +1079,7 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
         }
 
         /* CTCP ACTION */
-        if (strcmp (arguments + 1, "ACTION") == 0)
+        if (weechat_strcasecmp (ptr_args + 1, "action") == 0)
         {
             nick_is_me = (irc_server_strcasecmp (server, server->nick, nick) == 0);
             if (channel)
@@ -1011,22 +1093,67 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
                 irc_channel_nick_speaking_time_remove_old (channel);
                 irc_channel_nick_speaking_time_add (server, channel, nick,
                                                     time (NULL));
-                weechat_printf_date_tags (
-                    channel->buffer,
-                    date,
-                    irc_protocol_tags (command,
-                                       (nick_is_me) ?
-                                       "irc_action,notify_none,no_highlight" :
-                                       "irc_action,notify_message",
-                                       nick, address),
-                    "%s%s%s%s%s%s%s",
-                    weechat_prefix ("action"),
-                    irc_nick_mode_for_display (server, ptr_nick, 0),
-                    (ptr_nick) ? ptr_nick->color : ((nick) ? irc_nick_find_color (nick) : IRC_COLOR_CHAT_NICK),
-                    nick,
-                    (pos_args) ? IRC_COLOR_RESET : "",
-                    (pos_args) ? " " : "",
-                    (pos_args) ? pos_args : "");
+                if (ptr_nick)
+                    nick_color = strdup (ptr_nick->color);
+                else if (nick)
+                    nick_color = irc_nick_find_color (nick);
+                else
+                    nick_color = strdup (IRC_COLOR_CHAT_NICK);
+                if (irc_server_prefix_char_statusmsg (server, target[0]))
+                {
+                    /* STATUSMSG action */
+                    weechat_printf_date_tags (
+                        channel->buffer,
+                        date,
+                        irc_protocol_tags (
+                            server,
+                            command,
+                            tags,
+                            (nick_is_me) ?
+                            "irc_action,self_msg,notify_none,no_highlight" :
+                            "irc_action,notify_message",
+                            nick,
+                            address),
+                        "%s%s -> %s%s%s: %s%s%s%s%s%s",
+                        weechat_prefix ("network"),
+                        /* TRANSLATORS: "Action" is an IRC CTCP "ACTION" sent with /me */
+                        _("Action"),
+                        IRC_COLOR_CHAT_CHANNEL,
+                        target,
+                        IRC_COLOR_RESET,
+                        irc_nick_mode_for_display (server, ptr_nick, 0),
+                        nick_color,
+                        nick,
+                        (pos_args) ? IRC_COLOR_RESET : "",
+                        (pos_args) ? " " : "",
+                        (pos_args) ? pos_args : "");
+                }
+                else
+                {
+                    /* standard action */
+                    weechat_printf_date_tags (
+                        channel->buffer,
+                        date,
+                        irc_protocol_tags (
+                            server,
+                            command,
+                            tags,
+                            (nick_is_me) ?
+                            "irc_action,self_msg,notify_none,no_highlight" :
+                            "irc_action,notify_message",
+                            nick,
+                            address),
+                        "%s%s%s%s%s%s%s",
+                        weechat_prefix ("action"),
+                        irc_nick_mode_for_display (server, ptr_nick, 0),
+                        nick_color,
+                        nick,
+                        (pos_args) ? IRC_COLOR_RESET : "",
+                        (pos_args) ? " " : "",
+                        (pos_args) ? pos_args : "");
+                }
+                if (nick_color)
+                    free (nick_color);
             }
             else
             {
@@ -1053,11 +1180,14 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
                     weechat_printf_date_tags (
                         ptr_channel->buffer,
                         date,
-                        irc_protocol_tags (command,
-                                           (nick_is_me) ?
-                                           "irc_action,notify_none,no_highlight" :
-                                           "irc_action,notify_private",
-                                           nick, address),
+                        irc_protocol_tags (
+                            server,
+                            command,
+                            tags,
+                            (nick_is_me) ?
+                            "irc_action,self_msg,notify_none,no_highlight" :
+                            "irc_action,notify_private",
+                            nick, address),
                         "%s%s%s%s%s%s",
                         weechat_prefix ("action"),
                         (nick_is_me) ?
@@ -1068,35 +1198,50 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
                         (pos_args) ? pos_args : "");
                     (void) weechat_hook_signal_send ("irc_pv",
                                                      WEECHAT_HOOK_SIGNAL_STRING,
-                                                     message);
+                                                     (void *)message);
                 }
             }
         }
         /* CTCP PING */
-        else if (strcmp (arguments + 1, "PING") == 0)
+        else if (weechat_strcasecmp (ptr_args + 1, "ping") == 0)
         {
-            reply = irc_ctcp_get_reply (server, arguments + 1);
-            irc_ctcp_display_request (server, date, command, channel, nick,
-                                      address, arguments + 1, pos_args, reply);
+            reply = irc_ctcp_get_reply (server, ptr_args + 1);
+            irc_ctcp_display_request (server, date, tags, command, channel,
+                                      nick, address, ptr_args + 1, pos_args,
+                                      reply);
             if (!reply || reply[0])
             {
-                irc_ctcp_reply_to_nick (server, command, channel, nick,
-                                        arguments + 1, pos_args);
+                if (reply)
+                {
+                    decoded_reply = irc_ctcp_replace_variables (server, reply);
+                    if (decoded_reply)
+                    {
+                        irc_ctcp_reply_to_nick (server, tags, command, channel,
+                                                nick, ptr_args + 1,
+                                                decoded_reply);
+                        free (decoded_reply);
+                    }
+                }
+                else
+                {
+                    irc_ctcp_reply_to_nick (server, tags, command, channel,
+                                            nick, ptr_args + 1, pos_args);
+                }
             }
         }
         /* CTCP DCC */
-        else if (strcmp (arguments + 1, "DCC") == 0)
+        else if (weechat_strcasecmp (ptr_args + 1, "dcc") == 0)
         {
             irc_ctcp_recv_dcc (server, nick, pos_args, message);
         }
         /* other CTCP */
         else
         {
-            reply = irc_ctcp_get_reply (server, arguments + 1);
+            reply = irc_ctcp_get_reply (server, ptr_args + 1);
             if (reply)
             {
-                irc_ctcp_display_request (server, date, command, channel, nick,
-                                          address, arguments + 1, pos_args,
+                irc_ctcp_display_request (server, date, tags, command, channel,
+                                          nick, address, ptr_args + 1, pos_args,
                                           reply);
 
                 if (reply[0])
@@ -1104,8 +1249,9 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
                     decoded_reply = irc_ctcp_replace_variables (server, reply);
                     if (decoded_reply)
                     {
-                        irc_ctcp_reply_to_nick (server, command, channel, nick,
-                                                arguments + 1, decoded_reply);
+                        irc_ctcp_reply_to_nick (server, tags, command, channel,
+                                                nick, ptr_args + 1,
+                                                decoded_reply);
                         free (decoded_reply);
                     }
                 }
@@ -1119,14 +1265,15 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
                             server, nick, NULL, "ctcp",
                             (channel) ? channel->buffer : NULL),
                         date,
-                        irc_protocol_tags (command, "irc_ctcp", NULL, address),
+                        irc_protocol_tags (server, command, tags, "irc_ctcp",
+                                           NULL, address),
                         _("%sUnknown CTCP requested by %s%s%s: %s%s%s%s%s"),
                         weechat_prefix ("network"),
                         irc_nick_color_for_msg (server, 0, NULL, nick),
                         nick,
                         IRC_COLOR_RESET,
                         IRC_COLOR_CHAT_CHANNEL,
-                        arguments + 1,
+                        ptr_args + 1,
                         (pos_args) ? IRC_COLOR_RESET : "",
                         (pos_args) ? " " : "",
                         (pos_args) ? pos_args : "");
@@ -1136,14 +1283,27 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
 
         (void) weechat_hook_signal_send ("irc_ctcp",
                                          WEECHAT_HOOK_SIGNAL_STRING,
-                                         message);
+                                         (void *)message);
 
-        if (pos_space)
-            pos_space[0] = ' ';
-
-        if (pos_end)
-            pos_end[0] = '\01';
-
-        arguments = (pos_end) ? pos_end + 1 : NULL;
+        ptr_args = (pos_end) ? pos_end + 1 : NULL;
     }
+
+    free (dup_arguments);
+}
+
+/*
+ * Sends a CTCP to a target.
+ */
+
+void
+irc_ctcp_send (struct t_irc_server *server,
+               const char *target, const char *type, const char *args)
+{
+    irc_server_sendf (server,
+                      IRC_SERVER_SEND_OUTQ_PRIO_HIGH, NULL,
+                      "PRIVMSG %s :\01%s%s%s\01",
+                      target,
+                      type,
+                      (args) ? " " : "",
+                      (args) ? args : "");
 }
