@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2012 Simon Arlott
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -15,52 +15,75 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#ifndef WEECHAT_PLUGIN_SCRIPT_API_H
-#define WEECHAT_PLUGIN_SCRIPT_API_H 1
+#ifndef WEECHAT_PLUGIN_PLUGIN_SCRIPT_API_H
+#define WEECHAT_PLUGIN_PLUGIN_SCRIPT_API_H
+
+#include <time.h>
 
 extern void plugin_script_api_charset_set (struct t_plugin_script *script,
                                            const char *charset);
+extern int plugin_script_api_string_match_list (struct t_weechat_plugin *weechat_plugin,
+                                                const char *string,
+                                                const char *masks,
+                                                int case_sensitive);
 extern struct t_config_file *plugin_script_api_config_new (struct t_weechat_plugin *weechat_plugin,
                                                            struct t_plugin_script *script,
                                                            const char *name,
-                                                           int (*callback_reload)(void *data,
+                                                           int (*callback_reload)(const void *pointer,
+                                                                                  void *data,
                                                                                   struct t_config_file *config_file),
                                                            const char *function,
                                                            const char *data);
+extern int plugin_script_api_config_set_version (struct t_weechat_plugin *weechat_plugin,
+                                                 struct t_plugin_script *script,
+                                                 struct t_config_file *config_file,
+                                                 int version,
+                                                 struct t_hashtable *(*callback_update)(const void *pointer,
+                                                                                        void *data,
+                                                                                        struct t_config_file *config_file,
+                                                                                        int version_read,
+                                                                                        struct t_hashtable *data_read),
+                                                 const char *function,
+                                                 const char *data);
 extern struct t_config_section *plugin_script_api_config_new_section (struct t_weechat_plugin *weechat_plugin,
                                                                       struct t_plugin_script *script,
                                                                       struct t_config_file *config_file,
                                                                       const char *name,
                                                                       int user_can_add_options,
                                                                       int user_can_delete_options,
-                                                                      int (*callback_read)(void *data,
+                                                                      int (*callback_read)(const void *pointer,
+                                                                                           void *data,
                                                                                            struct t_config_file *config_file,
                                                                                            struct t_config_section *section,
                                                                                            const char *option_name,
                                                                                            const char *value),
                                                                       const char *function_read,
                                                                       const char *data_read,
-                                                                      int (*callback_write)(void *data,
+                                                                      int (*callback_write)(const void *pointer,
+                                                                                            void *data,
                                                                                             struct t_config_file *config_file,
                                                                                             const char *section_name),
                                                                       const char *function_write,
                                                                       const char *data_write,
-                                                                      int (*callback_write_default)(void *data,
+                                                                      int (*callback_write_default)(const void *pointer,
+                                                                                                    void *data,
                                                                                                     struct t_config_file *config_file,
                                                                                                     const char *section_name),
                                                                       const char *function_write_default,
                                                                       const char *data_write_default,
-                                                                      int (*callback_create_option)(void *data,
+                                                                      int (*callback_create_option)(const void *pointer,
+                                                                                                    void *data,
                                                                                                     struct t_config_file *config_file,
                                                                                                     struct t_config_section *section,
                                                                                                     const char *option_name,
                                                                                                     const char *value),
                                                                       const char *function_create_option,
                                                                       const char *data_create_option,
-                                                                      int (*callback_delete_option)(void *data,
+                                                                      int (*callback_delete_option)(const void *pointer,
+                                                                                                    void *data,
                                                                                                     struct t_config_file *config_file,
                                                                                                     struct t_config_section *section,
                                                                                                     struct t_config_option *option),
@@ -78,31 +101,22 @@ extern struct t_config_option *plugin_script_api_config_new_option (struct t_wee
                                                                     const char *default_value,
                                                                     const char *value,
                                                                     int null_value_allowed,
-                                                                    int (*callback_check_value)(void *data,
+                                                                    int (*callback_check_value)(const void *pointer,
+                                                                                                void *data,
                                                                                                 struct t_config_option *option,
                                                                                                 const char *value),
                                                                     const char *function_check_value,
                                                                     const char *data_check_value,
-                                                                    void (*callback_change)(void *data,
+                                                                    void (*callback_change)(const void *pointer,
+                                                                                            void *data,
                                                                                             struct t_config_option *option),
                                                                     const char *function_change,
                                                                     const char *data_change,
-                                                                    void (*callback_delete)(void *data,
+                                                                    void (*callback_delete)(const void *pointer,
+                                                                                            void *data,
                                                                                             struct t_config_option *option),
                                                                     const char *function_delete,
                                                                     const char *data_delete);
-extern void plugin_script_api_config_option_free (struct t_weechat_plugin *weechat_plugin,
-                                                  struct t_plugin_script *script,
-                                                  struct t_config_option *option);
-extern void plugin_script_api_config_section_free_options (struct t_weechat_plugin *weechat_plugin,
-                                                           struct t_plugin_script *script,
-                                                           struct t_config_section *section);
-extern void plugin_script_api_config_section_free (struct t_weechat_plugin *weechat_plugin,
-                                                   struct t_plugin_script *script,
-                                                   struct t_config_section *section);
-extern void plugin_script_api_config_free (struct t_weechat_plugin *weechat_plugin,
-                                           struct t_plugin_script *script,
-                                           struct t_config_file *config_file);
 extern void plugin_script_api_printf (struct t_weechat_plugin *weechat_plugin,
                                       struct t_plugin_script *script,
                                       struct t_gui_buffer *buffer,
@@ -116,6 +130,12 @@ extern void plugin_script_api_printf_y (struct t_weechat_plugin *weechat_plugin,
                                         struct t_plugin_script *script,
                                         struct t_gui_buffer *buffer,
                                         int y, const char *format, ...);
+extern void plugin_script_api_printf_y_date_tags (struct t_weechat_plugin *weechat_plugin,
+                                                  struct t_plugin_script *script,
+                                                  struct t_gui_buffer *buffer,
+                                                  int y, time_t date,
+                                                  const char *tags,
+                                                  const char *format, ...);
 extern void plugin_script_api_log_printf (struct t_weechat_plugin *weechat_plugin,
                                           struct t_plugin_script *script,
                                           const char *format, ...);
@@ -124,7 +144,8 @@ extern struct t_hook *plugin_script_api_hook_command (struct t_weechat_plugin *w
                                                       const char *command, const char *description,
                                                       const char *args, const char *args_description,
                                                       const char *completion,
-                                                      int (*callback)(void *data,
+                                                      int (*callback)(const void *pointer,
+                                                                      void *data,
                                                                       struct t_gui_buffer *buffer,
                                                                       int argc, char **argv,
                                                                       char **argv_eol),
@@ -133,16 +154,18 @@ extern struct t_hook *plugin_script_api_hook_command (struct t_weechat_plugin *w
 extern struct t_hook *plugin_script_api_hook_command_run (struct t_weechat_plugin *weechat_plugin,
                                                           struct t_plugin_script *script,
                                                           const char *command,
-                                                          int (*callback)(void *data,
+                                                          int (*callback)(const void *pointer,
+                                                                          void *data,
                                                                           struct t_gui_buffer *buffer,
                                                                           const char *command),
                                                           const char *function,
                                                           const char *data);
 extern struct t_hook *plugin_script_api_hook_timer (struct t_weechat_plugin *weechat_plugin,
                                                     struct t_plugin_script *script,
-                                                    int interval, int align_second,
+                                                    long interval, int align_second,
                                                     int max_calls,
-                                                    int (*callback)(void *data,
+                                                    int (*callback)(const void *pointer,
+                                                                    void *data,
                                                                     int remaining_calls),
                                                     const char *function,
                                                     const char *data);
@@ -150,7 +173,9 @@ extern struct t_hook *plugin_script_api_hook_fd (struct t_weechat_plugin *weecha
                                                  struct t_plugin_script *script,
                                                  int fd, int flag_read,
                                                  int flag_write, int flag_exception,
-                                                 int (*callback)(void *data, int fd),
+                                                 int (*callback)(const void *pointer,
+                                                                 void *data,
+                                                                 int fd),
                                                  const char *function,
                                                  const char *data);
 extern struct t_hook *plugin_script_api_hook_process_hashtable (struct t_weechat_plugin *weechat_plugin,
@@ -158,7 +183,8 @@ extern struct t_hook *plugin_script_api_hook_process_hashtable (struct t_weechat
                                                                 const char *command,
                                                                 struct t_hashtable *options,
                                                                 int timeout,
-                                                                int (*callback)(void *data,
+                                                                int (*callback)(const void *pointer,
+                                                                                void *data,
                                                                                 const char *command,
                                                                                 int return_code,
                                                                                 const char *out,
@@ -169,7 +195,8 @@ extern struct t_hook *plugin_script_api_hook_process (struct t_weechat_plugin *w
                                                       struct t_plugin_script *script,
                                                       const char *command,
                                                       int timeout,
-                                                      int (*callback)(void *data,
+                                                      int (*callback)(const void *pointer,
+                                                                      void *data,
                                                                       const char *command,
                                                                       int return_code,
                                                                       const char *out,
@@ -188,7 +215,8 @@ extern struct t_hook *plugin_script_api_hook_connect (struct t_weechat_plugin *w
                                                       int gnutls_dhkey_size,
                                                       const char *gnutls_priorities,
                                                       const char *local_hostname,
-                                                      int (*callback)(void *data,
+                                                      int (*callback)(const void *pointer,
+                                                                      void *data,
                                                                       int status,
                                                                       int gnutls_rc,
                                                                       int sock,
@@ -196,13 +224,24 @@ extern struct t_hook *plugin_script_api_hook_connect (struct t_weechat_plugin *w
                                                                       const char *ip_address),
                                                       const char *function,
                                                       const char *data);
+extern struct t_hook *plugin_script_api_hook_line (struct t_weechat_plugin *weechat_plugin,
+                                                   struct t_plugin_script *script,
+                                                   const char *buffer_type,
+                                                   const char *buffer_name,
+                                                   const char *tags,
+                                                   struct t_hashtable *(*callback)(const void *pointer,
+                                                                                   void *data,
+                                                                                   struct t_hashtable *line),
+                                                   const char *function,
+                                                   const char *data);
 extern struct t_hook *plugin_script_api_hook_print (struct t_weechat_plugin *weechat_plugin,
                                                     struct t_plugin_script *script,
                                                     struct t_gui_buffer *buffer,
                                                     const char *tags,
                                                     const char *message,
                                                     int strip_colors,
-                                                    int (*callback)(void *data,
+                                                    int (*callback)(const void *pointer,
+                                                                    void *data,
                                                                     struct t_gui_buffer *buffer,
                                                                     time_t date,
                                                                     int tags_count,
@@ -216,7 +255,8 @@ extern struct t_hook *plugin_script_api_hook_print (struct t_weechat_plugin *wee
 extern struct t_hook *plugin_script_api_hook_signal (struct t_weechat_plugin *weechat_plugin,
                                                      struct t_plugin_script *script,
                                                      const char *signal,
-                                                     int (*callback)(void *data,
+                                                     int (*callback)(const void *pointer,
+                                                                     void *data,
                                                                      const char *signal,
                                                                      const char *type_data,
                                                                      void *signal_data),
@@ -225,7 +265,8 @@ extern struct t_hook *plugin_script_api_hook_signal (struct t_weechat_plugin *we
 extern struct t_hook *plugin_script_api_hook_hsignal (struct t_weechat_plugin *weechat_plugin,
                                                       struct t_plugin_script *script,
                                                       const char *signal,
-                                                      int (*callback)(void *data,
+                                                      int (*callback)(const void *pointer,
+                                                                      void *data,
                                                                       const char *signal,
                                                                       struct t_hashtable *hashtable),
                                                       const char *function,
@@ -233,7 +274,8 @@ extern struct t_hook *plugin_script_api_hook_hsignal (struct t_weechat_plugin *w
 extern struct t_hook *plugin_script_api_hook_config (struct t_weechat_plugin *weechat_plugin,
                                                      struct t_plugin_script *script,
                                                      const char *option,
-                                                     int (*callback)(void *data,
+                                                     int (*callback)(const void *pointer,
+                                                                     void *data,
                                                                      const char *option,
                                                                      const char *value),
                                                      const char *function,
@@ -242,7 +284,8 @@ extern struct t_hook *plugin_script_api_hook_completion (struct t_weechat_plugin
                                                          struct t_plugin_script *script,
                                                          const char *completion,
                                                          const char *description,
-                                                         int (*callback)(void *data,
+                                                         int (*callback)(const void *pointer,
+                                                                         void *data,
                                                                          const char *completion_item,
                                                                          struct t_gui_buffer *buffer,
                                                                          struct t_gui_completion *completion),
@@ -251,7 +294,8 @@ extern struct t_hook *plugin_script_api_hook_completion (struct t_weechat_plugin
 extern struct t_hook *plugin_script_api_hook_modifier (struct t_weechat_plugin *weechat_plugin,
                                                        struct t_plugin_script *script,
                                                        const char *modifier,
-                                                       char *(*callback)(void *data,
+                                                       char *(*callback)(const void *pointer,
+                                                                         void *data,
                                                                          const char *modifier,
                                                                          const char *modifier_data,
                                                                          const char *string),
@@ -262,9 +306,10 @@ extern struct t_hook *plugin_script_api_hook_info (struct t_weechat_plugin *weec
                                                    const char *info_name,
                                                    const char *description,
                                                    const char *args_description,
-                                                   const char *(*callback)(void *data,
-                                                                           const char *info_name,
-                                                                           const char *arguments),
+                                                   char *(*callback)(const void *pointer,
+                                                                     void *data,
+                                                                     const char *info_name,
+                                                                     const char *arguments),
                                                    const char *function,
                                                    const char *data);
 extern struct t_hook *plugin_script_api_hook_info_hashtable (struct t_weechat_plugin *weechat_plugin,
@@ -273,7 +318,8 @@ extern struct t_hook *plugin_script_api_hook_info_hashtable (struct t_weechat_pl
                                                              const char *description,
                                                              const char *args_description,
                                                              const char *output_description,
-                                                             struct t_hashtable *(*callback)(void *data,
+                                                             struct t_hashtable *(*callback)(const void *pointer,
+                                                                                             void *data,
                                                                                              const char *info_name,
                                                                                              struct t_hashtable *hashtable),
                                                              const char *function,
@@ -284,52 +330,66 @@ extern struct t_hook *plugin_script_api_hook_infolist (struct t_weechat_plugin *
                                                        const char *description,
                                                        const char *pointer_description,
                                                        const char *args_description,
-                                                       struct t_infolist *(*callback)(void *data,
+                                                       struct t_infolist *(*callback)(const void *pointer,
+                                                                                      void *data,
                                                                                       const char *infolist_name,
-                                                                                      void *pointer,
+                                                                                      void *obj_pointer,
                                                                                       const char *arguments),
                                                        const char *function,
                                                        const char *data);
 extern struct t_hook *plugin_script_api_hook_focus (struct t_weechat_plugin *weechat_plugin,
                                                     struct t_plugin_script *script,
                                                     const char *area,
-                                                    struct t_hashtable *(*callback)(void *data,
+                                                    struct t_hashtable *(*callback)(const void *pointer,
+                                                                                    void *data,
                                                                                     struct t_hashtable *info),
                                                     const char *function,
                                                     const char *data);
-extern void plugin_script_api_unhook (struct t_weechat_plugin *weechat_plugin,
-                                      struct t_plugin_script *script,
-                                      struct t_hook *hook);
-extern void plugin_script_api_unhook_all (struct t_weechat_plugin *weechat_plugin,
-                                          struct t_plugin_script *script);
+extern struct t_gui_buffer *plugin_script_api_buffer_new_props (struct t_weechat_plugin *weechat_plugin,
+                                                                struct t_plugin_script *script,
+                                                                const char *name,
+                                                                struct t_hashtable *properties,
+                                                                int (*input_callback)(const void *pointer,
+                                                                                      void *data,
+                                                                                      struct t_gui_buffer *buffer,
+                                                                                      const char *input_data),
+                                                                const char *function_input,
+                                                                const char *data_input,
+                                                                int (*close_callback)(const void *pointer,
+                                                                                      void *data,
+                                                                                      struct t_gui_buffer *buffer),
+                                                                const char *function_close,
+                                                                const char *data_close);
 extern struct t_gui_buffer *plugin_script_api_buffer_new (struct t_weechat_plugin *weechat_plugin,
                                                           struct t_plugin_script *script,
                                                           const char *name,
-                                                          int (*input_callback)(void *data,
+                                                          int (*input_callback)(const void *pointer,
+                                                                                void *data,
                                                                                 struct t_gui_buffer *buffer,
                                                                                 const char *input_data),
                                                           const char *function_input,
                                                           const char *data_input,
-                                                          int (*close_callback)(void *data,
+                                                          int (*close_callback)(const void *pointer,
+                                                                                void *data,
                                                                                 struct t_gui_buffer *buffer),
                                                           const char *function_close,
                                                           const char *data_close);
-extern void plugin_script_api_buffer_close (struct t_weechat_plugin *weechat_plugin,
-                                            struct t_plugin_script *script,
-                                            struct t_gui_buffer *buffer);
 extern struct t_gui_bar_item *plugin_script_api_bar_item_new (struct t_weechat_plugin *weechat_plugin,
                                                               struct t_plugin_script *script,
                                                               const char *name,
-                                                              char *(*build_callback)(void *data,
+                                                              char *(*build_callback)(const void *pointer,
+                                                                                      void *data,
                                                                                       struct t_gui_bar_item *item,
                                                                                       struct t_gui_window *window,
                                                                                       struct t_gui_buffer *buffer,
                                                                                       struct t_hashtable *extra_info),
                                                               const char *function,
                                                               const char *data);
-extern void plugin_script_api_bar_item_remove (struct t_weechat_plugin *weechat_plugin,
-                                               struct t_plugin_script *script,
-                                               struct t_gui_bar_item *item);
+extern int plugin_script_api_command_options (struct t_weechat_plugin *weechat_plugin,
+                                              struct t_plugin_script *script,
+                                              struct t_gui_buffer *buffer,
+                                              const char *command,
+                                              struct t_hashtable *options);
 extern int plugin_script_api_command (struct t_weechat_plugin *weechat_plugin,
                                       struct t_plugin_script *script,
                                       struct t_gui_buffer *buffer,
@@ -350,14 +410,15 @@ extern void plugin_script_api_config_set_desc_plugin (struct t_weechat_plugin *w
 extern int plugin_script_api_config_unset_plugin (struct t_weechat_plugin *weechat_plugin,
                                                   struct t_plugin_script *script,
                                                   const char *option);
-extern int plugin_script_api_upgrade_read (struct t_weechat_plugin *weechat_plugin,
-                                           struct t_plugin_script *script,
-                                           struct t_upgrade_file *upgrade_file,
-                                           int (*callback_read)(void *data,
-                                                                struct t_upgrade_file *upgrade_file,
-                                                                int object_id,
-                                                                struct t_infolist *infolist),
-                                           const char *function,
-                                           const char *data);
+extern struct t_upgrade_file *plugin_script_api_upgrade_new (struct t_weechat_plugin *weechat_plugin,
+                                                             struct t_plugin_script *script,
+                                                             const char *filename,
+                                                             int (*callback_read)(const void *pointer,
+                                                                                  void *data,
+                                                                                  struct t_upgrade_file *upgrade_file,
+                                                                                  int object_id,
+                                                                                  struct t_infolist *infolist),
+                                                             const char *function,
+                                                             const char *data);
 
-#endif /* WEECHAT_PLUGIN_SCRIPT_API_H */
+#endif /* WEECHAT_PLUGIN_PLUGIN_SCRIPT_API_H */
