@@ -1,7 +1,7 @@
 /*
  * script-buffer.c - display scripts on script buffer
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -117,8 +117,9 @@ script_buffer_display_line_script (int line, struct t_script_repo *script)
                     if (script->date_added > 0)
                     {
                         tm = localtime (&script->date_added);
-                        strftime (str_date, sizeof (str_date),
-                                  "%Y-%m-%d", tm);
+                        if (strftime (str_date, sizeof (str_date),
+                                      "%Y-%m-%d", tm) == 0)
+                            str_date[0] = '\0';
                         snprintf (str_item, sizeof (str_item),
                                   "%s%s",
                                   weechat_color (
@@ -232,8 +233,9 @@ script_buffer_display_line_script (int line, struct t_script_repo *script)
                     if (script->date_updated > 0)
                     {
                         tm = localtime (&script->date_updated);
-                        strftime (str_date, sizeof (str_date),
-                                  "%Y-%m-%d", tm);
+                        if (strftime (str_date, sizeof (str_date),
+                                      "%Y-%m-%d", tm) == 0)
+                            str_date[0] = '\0';
                         snprintf (str_item, sizeof (str_item),
                                   "%s%s",
                                   weechat_color (
@@ -339,7 +341,7 @@ script_buffer_display_line_script (int line, struct t_script_repo *script)
 const char *
 script_buffer_detail_label (const char *text, int max_length)
 {
-    char str_format[16];
+    char str_format[128];
     static char result[1024];
     int num_spaces;
 
@@ -403,12 +405,9 @@ script_buffer_get_script_usage (struct t_script_repo *script)
     struct t_weelist *list;
     char hdata_name[128], str_option[256], str_info[1024];
     int config_files;
-    const char *ptr_name_hdata_callback, *type;
-    struct t_hdata *ptr_hdata_script, *ptr_hdata_callback;
-    struct t_hdata *ptr_hdata_config_file, *ptr_hdata_bar_item;
-    void *ptr_script, *ptr_callback;
-    struct t_config_file *ptr_config_file;
-    struct t_hook *ptr_hook;
+    struct t_hdata *hdata_script, *hdata_config, *hdata_bar_item;
+    void *ptr_script, *callback_pointer;
+    struct t_config_file *ptr_config;
     struct t_gui_bar_item *ptr_bar_item;
     struct t_infolist *infolist;
 
@@ -416,126 +415,161 @@ script_buffer_get_script_usage (struct t_script_repo *script)
 
     snprintf (hdata_name, sizeof (hdata_name),
               "%s_script", script_language[script->language]);
-    ptr_hdata_script = weechat_hdata_get (hdata_name);
-    if (!ptr_hdata_script)
+    hdata_script = weechat_hdata_get (hdata_name);
+    if (!hdata_script)
         return NULL;
 
-    ptr_script = script_buffer_get_script_pointer (script, ptr_hdata_script);
+    ptr_script = script_buffer_get_script_pointer (script, hdata_script);
     if (!ptr_script)
         return NULL;
 
-    ptr_name_hdata_callback = weechat_hdata_get_var_hdata (ptr_hdata_script,
-                                                           "callbacks");
-    if (!ptr_name_hdata_callback)
-        return NULL;
-    ptr_hdata_callback = weechat_hdata_get (ptr_name_hdata_callback);
-    if (!ptr_hdata_callback)
-        return NULL;
-
     list = weechat_list_new ();
 
-    ptr_hdata_config_file = weechat_hdata_get ("config_file");
-    ptr_hdata_bar_item = weechat_hdata_get ("bar_item");
-
-    ptr_callback = weechat_hdata_pointer (ptr_hdata_script,
-                                          ptr_script,
-                                          "callbacks");
-    while (ptr_callback)
+    /* get configuration files created by the script */
+    hdata_config = weechat_hdata_get ("config_file");
+    ptr_config = weechat_hdata_get_list (hdata_config, "config_files");
+    while (ptr_config)
     {
-        str_info[0] = '\0';
-        ptr_config_file = weechat_hdata_pointer (ptr_hdata_callback,
-                                                 ptr_callback,
-                                                 "config_file");
-        ptr_hook = weechat_hdata_pointer (ptr_hdata_callback,
-                                          ptr_callback,
-                                          "hook");
-        ptr_bar_item = weechat_hdata_pointer (ptr_hdata_callback,
-                                              ptr_callback,
-                                              "bar_item");
-        if (ptr_config_file)
+        callback_pointer = weechat_hdata_pointer (
+            hdata_config, ptr_config, "callback_reload_pointer");
+        if (callback_pointer == ptr_script)
         {
             snprintf (str_info, sizeof (str_info),
                       _("configuration file \"%s\" (options %s.*)"),
-                      weechat_hdata_string (ptr_hdata_config_file,
-                                            ptr_config_file,
+                      weechat_hdata_string (hdata_config, ptr_config,
                                             "filename"),
-                      weechat_hdata_string (ptr_hdata_config_file,
-                                            ptr_config_file,
+                      weechat_hdata_string (hdata_config, ptr_config,
                                             "name"));
+            weechat_list_add (list, str_info, WEECHAT_LIST_POS_END, NULL);
             config_files++;
         }
-        else if (ptr_hook)
+        ptr_config = weechat_hdata_move (hdata_config, ptr_config, 1);
+    }
+
+    /* get the commands created by the script */
+    infolist = weechat_infolist_get ("hook", NULL, "command");
+    if (infolist)
+    {
+        while (weechat_infolist_next (infolist))
         {
-            infolist = weechat_infolist_get ("hook", ptr_hook, NULL);
-            if (infolist)
+            callback_pointer = weechat_infolist_pointer (infolist,
+                                                         "callback_pointer");
+            if (callback_pointer == ptr_script)
             {
-                if (weechat_infolist_next (infolist))
-                {
-                    type = weechat_infolist_string (infolist, "type");
-                    if (type)
-                    {
-                        if (strcmp (type, "command") == 0)
-                        {
-                            snprintf (str_info, sizeof (str_info),
-                                      _("command /%s"),
-                                      weechat_infolist_string (infolist,
-                                                               "command"));
-                        }
-                        else if (strcmp (type, "completion") == 0)
-                        {
-                            snprintf (str_info, sizeof (str_info),
-                                      _("completion %%(%s)"),
-                                      weechat_infolist_string (infolist,
-                                                               "completion_item"));
-                        }
-                        else if (strcmp (type, "info") == 0)
-                        {
-                            snprintf (str_info, sizeof (str_info),
-                                      "info \"%s\"",
-                                      weechat_infolist_string (infolist,
-                                                               "info_name"));
-                        }
-                        else if (strcmp (type, "info_hashtable") == 0)
-                        {
-                            snprintf (str_info, sizeof (str_info),
-                                      "info_hashtable \"%s\"",
-                                      weechat_infolist_string (infolist,
-                                                               "info_name"));
-                        }
-                        else if (strcmp (type, "infolist") == 0)
-                        {
-                            snprintf (str_info, sizeof (str_info),
-                                      "infolist \"%s\"",
-                                      weechat_infolist_string (infolist,
-                                                               "infolist_name"));
-                        }
-                    }
-                }
-                weechat_infolist_free (infolist);
+                snprintf (str_info, sizeof (str_info),
+                          _("command /%s"),
+                          weechat_infolist_string (infolist,
+                                                   "command"));
+                weechat_list_add (list, str_info, WEECHAT_LIST_POS_END, NULL);
             }
         }
-        else if (ptr_bar_item)
+        weechat_infolist_free (infolist);
+    }
+
+    /* get the completions created by the script */
+    infolist = weechat_infolist_get ("hook", NULL, "completion");
+    if (infolist)
+    {
+        while (weechat_infolist_next (infolist))
+        {
+            callback_pointer = weechat_infolist_pointer (infolist,
+                                                         "callback_pointer");
+            if (callback_pointer == ptr_script)
+            {
+                snprintf (str_info, sizeof (str_info),
+                          _("completion %%(%s)"),
+                          weechat_infolist_string (infolist,
+                                                   "completion_item"));
+                weechat_list_add (list, str_info, WEECHAT_LIST_POS_END, NULL);
+            }
+        }
+        weechat_infolist_free (infolist);
+    }
+
+    /* get the infos created by the script */
+    infolist = weechat_infolist_get ("hook", NULL, "info");
+    if (infolist)
+    {
+        while (weechat_infolist_next (infolist))
+        {
+            callback_pointer = weechat_infolist_pointer (infolist,
+                                                         "callback_pointer");
+            if (callback_pointer == ptr_script)
+            {
+                snprintf (str_info, sizeof (str_info),
+                          "info \"%s\"",
+                          weechat_infolist_string (infolist,
+                                                   "info_name"));
+                weechat_list_add (list, str_info, WEECHAT_LIST_POS_END, NULL);
+            }
+        }
+        weechat_infolist_free (infolist);
+    }
+
+    /* get the infos (hashtable) created by the script */
+    infolist = weechat_infolist_get ("hook", NULL, "info_hashtable");
+    if (infolist)
+    {
+        while (weechat_infolist_next (infolist))
+        {
+            callback_pointer = weechat_infolist_pointer (infolist,
+                                                         "callback_pointer");
+            if (callback_pointer == ptr_script)
+            {
+                snprintf (str_info, sizeof (str_info),
+                          "info_hashtable \"%s\"",
+                          weechat_infolist_string (infolist,
+                                                   "info_name"));
+                weechat_list_add (list, str_info, WEECHAT_LIST_POS_END, NULL);
+            }
+        }
+        weechat_infolist_free (infolist);
+    }
+
+    /* get the infolists created by the script */
+    infolist = weechat_infolist_get ("hook", NULL, "infolist");
+    if (infolist)
+    {
+        while (weechat_infolist_next (infolist))
+        {
+            callback_pointer = weechat_infolist_pointer (infolist,
+                                                         "callback_pointer");
+            if (callback_pointer == ptr_script)
+            {
+                snprintf (str_info, sizeof (str_info),
+                          "infolist \"%s\"",
+                          weechat_infolist_string (infolist,
+                                                   "infolist_name"));
+                weechat_list_add (list, str_info, WEECHAT_LIST_POS_END, NULL);
+            }
+        }
+        weechat_infolist_free (infolist);
+    }
+
+    /* get the bar items created by the script */
+    hdata_bar_item = weechat_hdata_get ("bar_item");
+    ptr_bar_item = weechat_hdata_get_list (hdata_bar_item, "gui_bar_items");
+    while (ptr_bar_item)
+    {
+        callback_pointer = weechat_hdata_pointer (hdata_bar_item, ptr_bar_item,
+                                                  "build_callback_pointer");
+        if (callback_pointer == ptr_script)
         {
             snprintf (str_info, sizeof (str_info),
                       _("bar item \"%s\""),
-                      weechat_hdata_string (ptr_hdata_bar_item,
+                      weechat_hdata_string (hdata_bar_item,
                                             ptr_bar_item,
                                             "name"));
+            weechat_list_add (list, str_info, WEECHAT_LIST_POS_END, NULL);
         }
-        if (str_info[0])
-        {
-            weechat_list_add (list, str_info,
-                              WEECHAT_LIST_POS_END, NULL);
-        }
-        ptr_callback = weechat_hdata_move (ptr_hdata_callback,
-                                           ptr_callback,
-                                           1);
+        ptr_bar_item = weechat_hdata_move (hdata_bar_item, ptr_bar_item, 1);
     }
 
+    /* get the script options (in plugins.var) */
     snprintf (str_option, sizeof (str_option),
               "plugins.var.%s.%s.*",
               script_language[script->language],
-              weechat_hdata_string (ptr_hdata_script, ptr_script, "name"));
+              weechat_hdata_string (hdata_script, ptr_script, "name"));
     infolist = weechat_infolist_get ("option", NULL, str_option);
     if (infolist)
     {
@@ -567,8 +601,8 @@ script_buffer_display_detail_script (struct t_script_repo *script)
     char *labels[] = { N_("Script"), N_("Version"), N_("Version loaded"),
                        N_("Author"), N_("License"), N_("Description"),
                        N_("Tags"), N_("Status"),  N_("Date added"),
-                       N_("Date updated"), N_("URL"), N_("MD5"), N_("Requires"),
-                       N_("Min WeeChat"), N_("Max WeeChat"),
+                       N_("Date updated"), N_("URL"), N_("SHA-512"),
+                       N_("Requires"), N_("Min WeeChat"), N_("Max WeeChat"),
                        NULL };
     int i, length, max_length, line;
     struct t_weelist *list;
@@ -640,14 +674,16 @@ script_buffer_display_detail_script (struct t_script_repo *script)
     }
     line++;
     tm = localtime (&script->date_added);
-    strftime (str_time, sizeof (str_time), "%Y-%m-%d %H:%M:%S", tm);
+    if (strftime (str_time, sizeof (str_time), "%Y-%m-%d %H:%M:%S", tm) == 0)
+        str_time[0] = '\0';
     weechat_printf_y (script_buffer, line + 1,
                       "%s: %s",
                       script_buffer_detail_label (_(labels[line]), max_length),
                       str_time);
     line++;
     tm = localtime (&script->date_updated);
-    strftime (str_time, sizeof (str_time), "%Y-%m-%d %H:%M:%S", tm);
+    if (strftime (str_time, sizeof (str_time), "%Y-%m-%d %H:%M:%S", tm) == 0)
+        str_time[0] = '\0';
     weechat_printf_y (script_buffer, line + 1,
                       "%s: %s",
                       script_buffer_detail_label (_(labels[line]), max_length),
@@ -661,7 +697,7 @@ script_buffer_display_detail_script (struct t_script_repo *script)
     weechat_printf_y (script_buffer, line + 1,
                       "%s: %s",
                       script_buffer_detail_label (_(labels[line]), max_length),
-                      script->md5sum);
+                      (script->sha512sum) ? script->sha512sum : "-");
     line++;
     weechat_printf_y (script_buffer, line + 1,
                       "%s: %s",
@@ -892,13 +928,14 @@ script_buffer_check_line_outside_window ()
  */
 
 int
-script_buffer_window_scrolled_cb (void *data, const char *signal,
-                                  const char *type_data,
+script_buffer_window_scrolled_cb (const void *pointer, void *data,
+                                  const char *signal, const char *type_data,
                                   void *signal_data)
 {
     int start_line_y, chat_height, line;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) signal;
     (void) type_data;
@@ -936,7 +973,8 @@ script_buffer_window_scrolled_cb (void *data, const char *signal,
  */
 
 int
-script_buffer_input_cb (void *data, struct t_gui_buffer *buffer,
+script_buffer_input_cb (const void *pointer, void *data,
+                        struct t_gui_buffer *buffer,
                         const char *input_data)
 {
     char *actions[][2] = { { "A", "toggleautoload" },
@@ -953,6 +991,7 @@ script_buffer_input_cb (void *data, struct t_gui_buffer *buffer,
     int i;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     /* close buffer */
@@ -1010,9 +1049,11 @@ script_buffer_input_cb (void *data, struct t_gui_buffer *buffer,
  */
 
 int
-script_buffer_close_cb (void *data, struct t_gui_buffer *buffer)
+script_buffer_close_cb (const void *pointer, void *data,
+                        struct t_gui_buffer *buffer)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) buffer;
 
@@ -1049,21 +1090,23 @@ script_buffer_set_callbacks ()
 void
 script_buffer_set_keys ()
 {
-    char *keys[][2] = { { "meta-A",  "toggleautoload" },
-                        { "meta-l",  "load"           },
-                        { "meta-u",  "unload"         },
-                        { "meta-L",  "reload"         },
-                        { "meta-i",  "install"        },
-                        { "meta-r",  "remove"         },
-                        { "meta-h",  "hold"           },
-                        { "meta-v",  "show"           },
-                        { "meta-d",  "showdiff"       },
-                        { NULL,     NULL              } };
+    char *keys[][2] = {
+        { "meta-A", "toggleautoload" },
+        { "meta-l", "load"           },
+        { "meta-u", "unload"         },
+        { "meta-L", "reload"         },
+        { "meta-i", "install"        },
+        { "meta-r", "remove"         },
+        { "meta-h", "hold"           },
+        { "meta-v", "show"           },
+        { "meta-d", "showdiff"       },
+        { NULL, NULL },
+    };
     char str_key[64], str_command[64];
     int i;
 
-    weechat_buffer_set (script_buffer, "key_bind_meta2-A", "/script up");
-    weechat_buffer_set (script_buffer, "key_bind_meta2-B", "/script down");
+    weechat_buffer_set (script_buffer, "key_bind_up", "/script up");
+    weechat_buffer_set (script_buffer, "key_bind_down", "/script down");
     for (i = 0; keys[i][0]; i++)
     {
         if (weechat_config_boolean (script_config_look_use_keys))
@@ -1080,6 +1123,20 @@ script_buffer_set_keys ()
     }
 }
 
+/*
+ * Sets the local variable "filter" in the script buffer.
+ */
+
+void
+script_buffer_set_localvar_filter ()
+{
+    if (!script_buffer)
+        return;
+
+    weechat_buffer_set (script_buffer, "localvar_set_filter",
+                        (script_repo_filter) ? script_repo_filter : "*");
+}
+
 /*
  * Opens script buffer.
  */
@@ -1087,22 +1144,38 @@ script_buffer_set_keys ()
 void
 script_buffer_open ()
 {
-    if (!script_buffer)
+    struct t_hashtable *buffer_props;
+
+    if (script_buffer)
+        return;
+
+    buffer_props = weechat_hashtable_new (
+        32,
+        WEECHAT_HASHTABLE_STRING,
+        WEECHAT_HASHTABLE_STRING,
+        NULL, NULL);
+    if (buffer_props)
     {
-        script_buffer = weechat_buffer_new (SCRIPT_BUFFER_NAME,
-                                            &script_buffer_input_cb, NULL,
-                                            &script_buffer_close_cb, NULL);
-
-        /* failed to create buffer ? then exit */
-        if (!script_buffer)
-            return;
-
-        weechat_buffer_set (script_buffer, "type", "free");
-        weechat_buffer_set (script_buffer, "title", _("Scripts"));
-        script_buffer_set_keys ();
-        weechat_buffer_set (script_buffer, "localvar_set_type", "script");
-
-        script_buffer_selected_line = 0;
-        script_buffer_detail_script = NULL;
+        weechat_hashtable_set (buffer_props, "type", "free");
+        weechat_hashtable_set (buffer_props, "title", _("Scripts"));
+        weechat_hashtable_set (buffer_props, "localvar_set_type", "script");
     }
+
+    script_buffer = weechat_buffer_new_props (
+        SCRIPT_BUFFER_NAME,
+        buffer_props,
+        &script_buffer_input_cb, NULL, NULL,
+        &script_buffer_close_cb, NULL, NULL);
+
+    if (buffer_props)
+        weechat_hashtable_free (buffer_props);
+
+    if (!script_buffer)
+        return;
+
+    script_buffer_set_keys ();
+    script_buffer_set_localvar_filter ();
+
+    script_buffer_selected_line = 0;
+    script_buffer_detail_script = NULL;
 }
