@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,16 +14,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#ifndef WEECHAT_SCRIPT_H
-#define WEECHAT_SCRIPT_H 1
+#ifndef WEECHAT_PLUGIN_SCRIPT_H
+#define WEECHAT_PLUGIN_SCRIPT_H
 
 #define weechat_plugin weechat_script_plugin
 #define SCRIPT_PLUGIN_NAME "script"
+#define SCRIPT_PLUGIN_PRIORITY 3000
 
-#define SCRIPT_NUM_LANGUAGES 7
+#define SCRIPT_NUM_LANGUAGES 8
 
 extern struct t_weechat_plugin *weechat_script_plugin;
 
@@ -34,8 +35,9 @@ extern struct t_hashtable *script_loaded;
 
 extern int script_language_search (const char *language);
 extern int script_language_search_by_extension (const char *extension);
+extern int script_download_enabled (int display_error);
 extern char *script_build_download_url (const char *url);
 extern void script_get_loaded_plugins ();
 extern void script_get_scripts ();
 
-#endif /* WEECHAT_SCRIPT_H */
+#endif /* WEECHAT_PLUGIN_SCRIPT_H */
